package org.crosswire.common.swing.desktop;

import java.awt.Insets;
import java.awt.LayoutManager;
import javax.swing.JPanel;
import javax.swing.UIManager;


/**
 * A JPanel class where it's child components will paint on top of its border
 */
public class TabbedPanePanel extends JPanel 
{   
    public TabbedPanePanel() 
    {
        super();
        init();
    }
    
    public TabbedPanePanel(boolean isDoubleBuffered) 
    {
        super(isDoubleBuffered);
        init();
    }
    
    public TabbedPanePanel(LayoutManager layout) 
    {
        super(layout);
        init();
    }
    
    public TabbedPanePanel(LayoutManager layout, boolean isDoubleBuffered) 
    {
        super(layout, isDoubleBuffered);
        init();
    }
    
    private void init() 
    {
        this.setBorder(UIManager.getBorder("TabbedPanePanel.border"));
    }
    
    /**
     * If we setBorder on this JPanel, the border width will be part of the insets. 
     * We return an insets of 0 so the child components will paint on top of the
     * border. 
     */
    public Insets getInsets() 
    {
        return new Insets(0, 0, 0, 0);
    }
}